"""
Configs migration

Generation datetime: 2021-09-03 10:41:01.590474
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper, LimitationToModify

mh: DatabaseMigrationHelper

REVISION = "a35c7832"
DOWN_REVISION = "c11ebacb"

DEFAULT_RPC_SETTINGS = {"timeouts": {"connect_timeout": 20, "request_timeout": 60, "response_timeout": 60}}


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="DESCRIPTORS_CACHE",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "cache_enabled": {"type": "boolean"},
                "updating_cache_interval": {"type": "number", "exclusiveMinimum": 0},
                "matching_settings": {
                    "type": "object",
                    "properties": {"thread_count": {"type": "integer", "minimum": 0}},
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
            },
            "description": "python matcher descriptor caching settings",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "cache_enabled": {"type": "boolean"},
                "updating_cache_interval": {"type": "number", "exclusiveMinimum": 0},
                "matching_settings": {
                    "type": "object",
                    "properties": {"thread_count": {"type": "integer", "minimum": 0}},
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
                "rpc_settings": {
                    "type": "object",
                    "properties": {
                        "timeouts": {
                            "type": "object",
                            "properties": {
                                "connect_timeout": {"type": "integer", "minimum": 0},
                                "request_timeout": {"type": "integer", "minimum": 0},
                                "response_timeout": {"type": "integer", "minimum": 0},
                            },
                            "description": "matcher lib settings",
                            "additionalProperties": False,
                        }
                    },
                    "description": "matcher lib settings",
                    "additionalProperties": False,
                },
            },
            "description": "python matcher descriptor caching settings",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "cache_enabled": True,
            "updating_cache_interval": 2,
            "matching_settings": {"thread_count": 0},
        },
        newDefaultValue={
            "cache_enabled": True,
            "updating_cache_interval": 2,
            "matching_settings": {"thread_count": 0},
            "rpc_settings": DEFAULT_RPC_SETTINGS,
        },
        settingUpgradeFunction=lambda setting: {**setting, "rpc_settings": DEFAULT_RPC_SETTINGS},
        settingDowngradeFunction=lambda setting: {n: v for n, v in setting.items() if n != "rpc_settings"},
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-python-matcher"],
        newServices=["luna-python-matcher"],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
