"""
Configs migration

Generation datetime: 2023-06-05 12:49:22.424310
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "365f0ef5"
DOWN_REVISION = "13582cfd"


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for basic attributes estimation in handlers",
        newDescription="luna-remote-sdk settings for basic attributes estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_GAZE_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for gaze estimation in handlers",
        newDescription="luna-remote-sdk settings for gaze estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_LANDMARKS68_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face landmarks68 estimation in handlers",
        newDescription="luna-remote-sdk settings for face landmarks68 estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_GLASSES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for glasses estimation in handlers",
        newDescription="luna-remote-sdk settings for glasses estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_HUMAN_DETECTOR_SETTINGS",
        oldDescription="luna-handlers settings for human detection in handlers",
        newDescription="luna-remote-sdk settings for human detection in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_RUNTIME_SETTINGS",
        oldDescription="runtime settings for luna-handlers",
        newDescription="runtime settings for luna-remote-sdk",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_EYES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for eyes estimation in handlers",
        newDescription="luna-remote-sdk settings for eyes estimation in handlers",
        oldGroupNames=["luna-platform", "luna-handlers-estimators-settings"],
        newGroupNames=["luna-platform", "luna-handlers-estimators-settings"],
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_RED_EYES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for red eyes estimation in handlers",
        newDescription="luna-remote-sdk settings for red eyes estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_DETECTOR_TYPE",
        oldDescription="luna-handlers face detector type",
        newDescription="luna-remote-sdk face detector type",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FISHEYE_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for fisheye estimation in handlers",
        newDescription="luna-remote-sdk settings for fisheye estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face detection background estimation in handlers",
        newDescription="luna-remote-sdk settings for face detection background estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for eyebrow expression estimation in handlers",
        newDescription="luna-remote-sdk settings for eyebrow expression estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_DETECTOR_SETTINGS",
        oldDescription="luna-handlers settings for body detection in handlers",
        newDescription="luna-remote-sdk settings for body detection in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_WARP_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face warp estimation in handlers",
        newDescription="luna-remote-sdk settings for face warp estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for body attributes estimation in handlers",
        newDescription="luna-remote-sdk settings for body attributes estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_HEADWEAR_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for headwear estimation in handlers",
        newDescription="luna-remote-sdk settings for headwear estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_WARP_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for body warp estimation in handlers",
        newDescription="luna-remote-sdk settings for body warp estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_HEAD_POSE_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for head pose estimation in handlers",
        newDescription="luna-remote-sdk settings for head pose estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for body descriptor estimation in handlers",
        newDescription="luna-remote-sdk settings for body descriptor estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for mouth attributes estimation in handlers",
        newDescription="luna-remote-sdk settings for mouth attributes estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_LIVENESS_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for liveness estimation in handlers",
        newDescription="luna-remote-sdk settings for liveness estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_QUALITY_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for quality estimation in handlers",
        newDescription="luna-remote-sdk settings for quality estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_EMOTIONS_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for emotions estimation in handlers",
        newDescription="luna-remote-sdk settings for emotions estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_LANDMARKS5_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face landmarks5 estimation in handlers",
        newDescription="luna-remote-sdk settings for face landmarks5 estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_BODY_LANDMARKS_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for body landmarks estimation in handlers",
        newDescription="luna-remote-sdk settings for body landmarks estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face natural light estimation in handlers",
        newDescription="luna-remote-sdk settings for face natural light estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_PORTRAIT_STYLE_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for portrait style estimation in handlers",
        newDescription="luna-remote-sdk settings for portrait style estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_DETECTOR_SETTINGS",
        oldDescription="luna-handlers settings for face detector",
        newDescription="luna-remote-sdk settings for face detector",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for image color type estimation in handlers",
        newDescription="luna-remote-sdk settings for image color type estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for image orientation estimation in handlers",
        newDescription="luna-remote-sdk settings for image orientation estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for face descriptor estimation in handlers",
        newDescription="luna-remote-sdk settings for face descriptor estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_MASK_ESTIMATOR_SETTINGS",
        oldDescription="luna-handlers settings for mask estimation in handlers",
        newDescription="luna-remote-sdk settings for mask estimation in handlers",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_USE_AUTO_ROTATION",
        oldDescription="whether to try estimate image rotation angle and rotate to normal state",
        newDescription="whether to try estimate image rotation angle and rotate to normal state",
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-remote-sdk"],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []

tenants = [
    "BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "BODY_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
    "BODY_DETECTOR_SETTINGS",
    "BODY_LANDMARKS_ESTIMATOR_SETTINGS",
    "BODY_WARP_ESTIMATOR_SETTINGS",
    "EMOTIONS_ESTIMATOR_SETTINGS",
    "EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS",
    "EYES_ESTIMATOR_SETTINGS",
    "FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
    "FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS",
    "FACE_DETECTOR_SETTINGS",
    "FACE_LANDMARKS5_ESTIMATOR_SETTINGS",
    "FACE_LANDMARKS68_ESTIMATOR_SETTINGS",
    "FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS",
    "FACE_WARP_ESTIMATOR_SETTINGS",
    "FISHEYE_ESTIMATOR_SETTINGS",
    "GAZE_ESTIMATOR_SETTINGS",
    "GLASSES_ESTIMATOR_SETTINGS",
    "HEADWEAR_ESTIMATOR_SETTINGS",
    "HEAD_POSE_ESTIMATOR_SETTINGS",
    "HUMAN_DETECTOR_SETTINGS",
    "IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS",
    "IMAGE_ORIENTATION_ESTIMATOR_SETTINGS",
    "LIVENESS_ESTIMATOR_SETTINGS",
    "MASK_ESTIMATOR_SETTINGS",
    "MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "PORTRAIT_STYLE_ESTIMATOR_SETTINGS",
    "QUALITY_ESTIMATOR_SETTINGS",
    "RED_EYES_ESTIMATOR_SETTINGS",
    "RUNTIME_SETTINGS",
    "DETECTOR_TYPE",
    "USE_AUTO_ROTATION",
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_REMOTE_SDK_LIMITS",
        validationSchema={
            "type": "object",
            "properties": {
                "received_images_limit": {"type": "integer", "minimum": 1},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "received_images_limit": 8,
        },
        description="luna remote sdk limits",
        services=["luna-remote-sdk"],
        groupNames=["luna-platform"],
    )

    # Rename
    for name in tenants:
        mh.renameLimitation("LUNA_HANDLERS_" + name, "LUNA_REMOTE_SDK_" + name)

    mh.renameLimitation("LUNA_HANDLERS_LOAD_EXTERNAL_IMAGES_TIMEOUT", "FETCH_IMAGE_TIMEOUTS")

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_REMOTE_SDK_LIMITS")

    # Rename
    for name in tenants:
        mh.renameLimitation("LUNA_REMOTE_SDK_" + name, "LUNA_HANDLERS_" + name)

    mh.renameLimitation("FETCH_IMAGE_TIMEOUTS", "LUNA_HANDLERS_LOAD_EXTERNAL_IMAGES_TIMEOUT")
