"""
Configs migration

Generation datetime: 2023-11-22 11:22:32.182741
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "b9c5a47a"
DOWN_REVISION = "e7490433"


def upgradeLimits(setting):
    return setting | {
        "raw_event_detections_limit": 100,
        "raw_event_arrays_limit": 30,
        "result_candidate_limit": 100,
    }


def downgradeLimits(setting):
    setting.pop("raw_event_detections_limit", None)
    setting.pop("raw_event_arrays_limit", None)
    setting.pop("result_candidate_limit", None)
    return setting


schemaFixLimitations = [
    "LUNA_REMOTE_SDK_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_WARP_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_RED_EYES_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FISHEYE_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_BODY_LANDMARKS_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_MASK_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_GAZE_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_HUMAN_DETECTOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_LANDMARKS5_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_EMOTIONS_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_BODY_WARP_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_FACE_LANDMARKS68_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_HEADWEAR_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_EYES_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_QUALITY_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_HEAD_POSE_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
    "LUNA_REMOTE_SDK_GLASSES_ESTIMATOR_SETTINGS",
]

limitationsToModify: list[LimitationToModify] = [
    *(
        LimitationToModify(
            limitationName=limitationName,
            oldValidationSchema={
                "type": "object",
                "properties": {
                    "runtime_settings": {
                        "type": "object",
                        "properties": {
                            "device_class": {
                                "type": "string",
                                "enum": ["global", "cpu", "gpu"],
                            },
                            "optimal_batch_size": {"type": "integer", "minimum": 1},
                            "worker_count": {"type": "integer", "minimum": 1},
                        },
                    },
                    "additionalProperties": False,
                },
                "description": "settings for sdk estimator",
                "additionalProperties": False,
            },
            newValidationSchema={
                "type": "object",
                "properties": {
                    "runtime_settings": {
                        "type": "object",
                        "properties": {
                            "device_class": {
                                "type": "string",
                                "enum": ["global", "cpu", "gpu"],
                            },
                            "optimal_batch_size": {"type": "integer", "minimum": 1},
                            "worker_count": {"type": "integer", "minimum": 1},
                        },
                        "additionalProperties": False,
                    }
                },
                "description": "settings for sdk estimator",
                "additionalProperties": False,
            },
            settingUpgradeFunction=lambda x: x,
            settingDowngradeFunction=lambda x: x,
            # DON'T REMOVE SERVICES. Used in profiled migrations.
            oldServices=["luna-remote-sdk"],
            newServices=["luna-remote-sdk"],
        )
        for limitationName in schemaFixLimitations
    ),
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_LIMITS",
        oldValidationSchema={
            "type": "object",
            "properties": {"received_images_limit": {"type": "integer", "minimum": 1}},
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "received_images_limit": {"type": "integer", "minimum": 1},
                "raw_event_detections_limit": {"type": "integer", "minimum": 1},
                "raw_event_arrays_limit": {"type": "integer", "minimum": 1},
                "result_candidate_limit": {"type": "integer", "minimum": 1},
            },
            "additionalProperties": False,
        },
        oldDefaultValue={"received_images_limit": 8},
        newDefaultValue={
            "received_images_limit": 8,
            "raw_event_detections_limit": 100,
            "raw_event_arrays_limit": 30,
            "result_candidate_limit": 100,
        },
        settingUpgradeFunction=upgradeLimits,
        settingDowngradeFunction=downgradeLimits,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-remote-sdk"],
        newServices=["luna-remote-sdk"],
    ),
    LimitationToModify(
        limitationName="LUNA_ACCOUNTS_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "dsn": {
                    "type": "string",
                    "pattern": "^(?:(?P<user>.*?)(?::(?P<passwd>.*))@)?(?P<net>[^\\?\\/]*)(?:\\/(?P<dbname>[^\\?]*)(?:\\?(?P<params>[^\\?]*))?)?$",
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": ["db_type", "db_settings"],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "dsn": {
                    "type": "string",
                    "pattern": "^(?:(?P<user>.*?)(?::(?P<passwd>.*))@)?(?P<net>[^\\?\\/]*)(?:\\/(?P<dbname>[^\\?]*)(?:\\?(?P<params>[^\\?]*))?)?$",
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": ["db_type", "db_settings"],
            "type": "object",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_admin",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 5},
        },
        newDefaultValue={
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_accounts",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 5},
        },
        settingUpgradeFunction=lambda x: x,
        settingDowngradeFunction=lambda x: x,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-accounts"],
        newServices=["luna-accounts"],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_CONFIGURATOR_ACTIVE_PLUGINS",
        validationSchema={"type": "array", "items": {"type": "string"}},
        defaultValue=[],
        description="list active plugins of luna-configurator",
        services=[],
        groupNames=["luna-platform"],
    )
