"""
Configs migration

Generation datetime: 2024-01-22 16:42:30.675518
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "6a3d8839"
DOWN_REVISION = "658d2427"


def upgradeFunction(setting: dict):
    setting.pop("raw_event_detections_limit", None)
    setting.pop("raw_event_arrays_limit", None)
    setting.pop("result_candidate_limit", None)

    return setting


def downgradeFunction(setting: dict):
    setting["raw_event_detections_limit"] = 100
    setting["raw_event_arrays_limit"] = 30
    setting["result_candidate_limit"] = 100

    return setting


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_REMOTE_SDK_LIMITS",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "received_images_limit": {"type": "integer", "minimum": 1},
                "raw_event_detections_limit": {"type": "integer", "minimum": 1},
                "raw_event_arrays_limit": {"type": "integer", "minimum": 1},
                "result_candidate_limit": {"type": "integer", "minimum": 1},
            },
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "received_images_limit": {"type": "integer", "minimum": 1},
            },
            "additionalProperties": False,
        },
        oldDefaultValue={
            "received_images_limit": 8,
            "raw_event_detections_limit": 100,
            "raw_event_arrays_limit": 30,
            "result_candidate_limit": 100,
        },
        newDefaultValue={"received_images_limit": 8},
        oldDescription="luna remote sdk limits",
        newDescription="luna remote sdk limits",
        oldGroupNames=["luna-platform"],
        newGroupNames=["luna-platform"],
        settingUpgradeFunction=upgradeFunction,
        settingDowngradeFunction=downgradeFunction,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-remote-sdk"],
        newServices=["luna-remote-sdk"],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
