"""
Module contains settings class for luna-accounts
"""

from typing import Optional

from ..settings.classes import DBSetting as _DBSetting
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsAccounts(BasePythonServiceSettingsClass):
    """
    Settings class for luna-accounts
    Attributes:
        db(DBSetting): database settings
        storageTime(str): storage time setting
    """

    serviceName = "luna-accounts"

    def __init__(self):
        super().__init__()
        self.db: Optional[_DBSetting] = None
        self.storageTime: Optional[str] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.db = _DBSetting(self.serviceName, self["LUNA_ACCOUNTS_DB"])
        self.storageTime = self["storage_time"]
