"""
Module contains settings class for luna-remote-sdk service
"""

from dataclasses import dataclass
from typing import Optional

from ..settings.classes import (
    AdditionalServicesUsageSetting as _AdditionalServicesUsageSetting,
    BodyDetectorSetting as _BodyDetectorSetting,
    DeepfakeSettings as _DeepfakeSettings,
    EstimatorRuntimeSetting as _EstimatorRuntimeSetting,
    FaceDetector3Setting as _Detector3Setting,
    FaceQualitySettings as _FaceQualitySettings,
    ImageStoreAddressSettings as _ImageStoreAddressSettings,
    Liveness2Settings as _Liveness2Settings,
    RemoteSDKLimitsSettings as _RemoteSDKLimitsSettings,
    RuntimeSetting as _RuntimeSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
    VideoSettings as _VideoSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


@dataclass
class EstimatorsSettings:
    """
    Estimators settings collection
    """

    faceDetector: Optional[_EstimatorRuntimeSetting] = None
    gazeEstimator: Optional[_EstimatorRuntimeSetting] = None
    qualityEstimator: Optional[_EstimatorRuntimeSetting] = None
    mouthAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    emotionsEstimator: Optional[_EstimatorRuntimeSetting] = None
    basicAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    eyesAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    headPoseEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceDescriptorEstimator: Optional[_EstimatorRuntimeSetting] = None
    maskEstimator: Optional[_EstimatorRuntimeSetting] = None
    livenessEstimator: Optional[_EstimatorRuntimeSetting] = None
    deepfakeEstimator: Optional[_EstimatorRuntimeSetting] = None
    glassesEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceWarpEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceLandmarks68Estimator: Optional[_EstimatorRuntimeSetting] = None
    faceLandmarks5Estimator: Optional[_EstimatorRuntimeSetting] = None
    imageColorTypeEstimator: Optional[_EstimatorRuntimeSetting] = None
    headwearEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceNaturalLightEstimator: Optional[_EstimatorRuntimeSetting] = None
    fisheyeEstimator: Optional[_EstimatorRuntimeSetting] = None
    eyebrowExpressionEstimator: Optional[_EstimatorRuntimeSetting] = None
    redEyesEstimator: Optional[_EstimatorRuntimeSetting] = None
    portraitStyleEstimator: Optional[_EstimatorRuntimeSetting] = None
    faceDetectionBackgroundEstimator: Optional[_EstimatorRuntimeSetting] = None
    imageOrientationEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyDetector: Optional[_EstimatorRuntimeSetting] = None
    bodyDescriptorEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyWarpEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyLandmarksEstimator: Optional[_EstimatorRuntimeSetting] = None
    bodyAttributesEstimator: Optional[_EstimatorRuntimeSetting] = None
    humanDetector: Optional[_EstimatorRuntimeSetting] = None
    peopleCounter: Optional[_EstimatorRuntimeSetting] = None


class SettingsRemoteSDK(BasePythonServiceSettingsClass):
    """
    Settings class for luna-remote-sdk
    Attributes:
        licensesAddress(ServiceAddressSettings): luna-licenses address settings
    """

    serviceName = "luna-remote-sdk"

    def __init__(self):
        super().__init__()
        self.licensesAddress: Optional[_ServiceAddressSettings] = None
        self.videoSettings: Optional[_VideoSettings] = None
        self.defaultFaceDescriptorVersion: Optional[int] = None
        self.defaultHumanDescriptorVersion: Optional[int] = None
        self.detectorType: Optional[str] = None
        self.detector3Settings: Optional[_Detector3Setting] = None
        self.bodyDetectorSetting: Optional[_BodyDetectorSetting] = None
        self.runtimeSettings: Optional[_RuntimeSetting] = None
        self.estimatorsSettings: Optional[EstimatorsSettings] = None
        self.useAutoRotation: Optional[bool] = None
        self.livenessSettings: Optional[_Liveness2Settings] = None
        self.deepfakeSettings: Optional[_DeepfakeSettings] = None
        self.faceQualitySettings: Optional[_FaceQualitySettings] = None
        self.remoteSDKLimitsSettings: Optional[_RemoteSDKLimitsSettings] = None
        self.externalApiAddress: Optional[_ServiceAddressSettings] = None

        self.facesStorage: Optional[_ImageStoreAddressSettings] = None
        self.bodiesStorage: Optional[_ImageStoreAddressSettings] = None
        self.imagesStorage: Optional[_ImageStoreAddressSettings] = None
        self.fetchImgTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.additionalServicesUsage: Optional[_AdditionalServicesUsageSetting] = None
        self.facesStorageTimeouts: _ServiceTimeoutsSettings = None
        self.bodiesStorageTimeouts: _ServiceTimeoutsSettings = None
        self.imagesStorageTimeouts: _ServiceTimeoutsSettings = None
        self.useAutoRotation: Optional[bool] = None

    def applyEstimatorsSettings(self):
        """
        Deserialize loaded estimators settings to classes
        """
        self.estimatorsSettings = EstimatorsSettings()

        self.estimatorsSettings.faceDetector = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_DETECTOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_DETECTOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.gazeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_GAZE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_GAZE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.qualityEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_QUALITY_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_QUALITY_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.mouthAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_MOUTH_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.emotionsEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_EMOTIONS_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_EMOTIONS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.basicAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BASIC_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.eyesAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_EYES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_EYES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.headPoseEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_HEAD_POSE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_HEAD_POSE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceDescriptorEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_DESCRIPTOR_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.maskEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_MASK_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_MASK_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.livenessEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_LIVENESS_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_LIVENESS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.deepfakeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_DEEPFAKE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_DEEPFAKE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.glassesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_GLASSES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_GLASSES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceWarpEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_WARP_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_WARP_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceLandmarks68Estimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_LANDMARKS68_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_LANDMARKS68_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceLandmarks5Estimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_LANDMARKS5_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_LANDMARKS5_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.imageColorTypeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_IMAGE_COLOR_TYPE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.headwearEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_HEADWEAR_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_HEADWEAR_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceNaturalLightEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_NATURAL_LIGHT_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.fisheyeEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FISHEYE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FISHEYE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.eyebrowExpressionEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_EYEBROW_EXPRESSION_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.redEyesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_RED_EYES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_RED_EYES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.faceDetectionBackgroundEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_FACE_DETECTION_BACKGROUND_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.portraitStyleEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_PORTRAIT_STYLE_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_PORTRAIT_STYLE_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.imageOrientationEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_IMAGE_ORIENTATION_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyDetector = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_BODY_DETECTOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BODY_DETECTOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyDescriptorEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BODY_DESCRIPTOR_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyWarpEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_BODY_WARP_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BODY_WARP_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyLandmarksEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_BODY_LANDMARKS_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BODY_LANDMARKS_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.bodyAttributesEstimator = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_ATTRIBUTES_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_BODY_ATTRIBUTES_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.humanDetector = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_HUMAN_DETECTOR_SETTINGS",
            self["LUNA_REMOTE_SDK_HUMAN_DETECTOR_SETTINGS"]["runtime_settings"],
        )
        self.estimatorsSettings.peopleCounter = _EstimatorRuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_PEOPLE_COUNT_ESTIMATOR_SETTINGS",
            self["LUNA_REMOTE_SDK_PEOPLE_COUNT_ESTIMATOR_SETTINGS"]["runtime_settings"],
        )

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.licensesAddress = _ServiceAddressSettings(
            self.serviceName, "LUNA_LICENSES_ADDRESS", self["LUNA_LICENSES_ADDRESS"]
        )

        self.videoSettings = _VideoSettings(
            self.serviceName, "LUNA_REMOTE_SDK_VIDEO_SETTINGS", self["LUNA_REMOTE_SDK_VIDEO_SETTINGS"]
        )

        self.defaultFaceDescriptorVersion = self["DEFAULT_FACE_DESCRIPTOR_VERSION"]
        self.defaultHumanDescriptorVersion = self["DEFAULT_HUMAN_DESCRIPTOR_VERSION"]
        self.detectorType = self["LUNA_REMOTE_SDK_DETECTOR_TYPE"]
        self.detector3Settings = _Detector3Setting(
            self.serviceName, self["LUNA_REMOTE_SDK_FACE_DETECTOR_SETTINGS"]["estimator_settings"]
        )
        self.bodyDetectorSetting = _BodyDetectorSetting(
            self.serviceName, self["LUNA_REMOTE_SDK_BODY_DETECTOR_SETTINGS"]["estimator_settings"]
        )
        self.runtimeSettings = _RuntimeSetting(
            self.serviceName,
            "LUNA_REMOTE_SDK_RUNTIME_SETTINGS",
            self["LUNA_REMOTE_SDK_RUNTIME_SETTINGS"],
        )

        self.applyEstimatorsSettings()

        self.useAutoRotation = self["LUNA_REMOTE_SDK_USE_AUTO_ROTATION"]

        self.livenessSettings = _Liveness2Settings(
            self.serviceName,
            "LUNA_REMOTE_SDK_LIVENESS_SETTINGS",
            self["LUNA_REMOTE_SDK_LIVENESS_ESTIMATOR_SETTINGS"]["estimator_settings"],
        )
        self.deepfakeSettings = _DeepfakeSettings(
            self.serviceName,
            "LUNA_REMOTE_SDK_DEEPFAKE_SETTINGS",
            self["LUNA_REMOTE_SDK_DEEPFAKE_ESTIMATOR_SETTINGS"]["estimator_settings"],
        )
        self.faceQualitySettings = _FaceQualitySettings(
            self.serviceName,
            "FACE_QUALITY_SETTINGS",
            self["FACE_QUALITY_SETTINGS"],
        )
        self.externalApiAddress = _ServiceAddressSettings(
            self.serviceName,
            "EXTERNAL_LUNA_API_ADDRESS",
            self["EXTERNAL_LUNA_API_ADDRESS"],
        )
        self.facesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_ADDRESS"],
        )
        self.bodiesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_ADDRESS"],
        )
        self.imagesStorage = _ImageStoreAddressSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_ADDRESS",
            self["LUNA_IMAGE_STORE_IMAGES_ADDRESS"],
        )
        self.fetchImgTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "FETCH_IMAGE_TIMEOUTS",
            self["FETCH_IMAGE_TIMEOUTS"],
        )
        self.additionalServicesUsage = _AdditionalServicesUsageSetting(
            self.serviceName,
            self["ADDITIONAL_SERVICES_USAGE"],
        )
        self.facesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_FACES_SAMPLES_TIMEOUTS"],
        )
        self.bodiesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_BODIES_SAMPLES_TIMEOUTS"],
        )
        self.imagesStorageTimeouts = _ServiceTimeoutsSettings(
            self.serviceName,
            "LUNA_IMAGE_STORE_IMAGES_TIMEOUTS",
            self["LUNA_IMAGE_STORE_IMAGES_TIMEOUTS"],
        )
        self.remoteSDKLimitsSettings = _RemoteSDKLimitsSettings(self.serviceName, self["LUNA_REMOTE_SDK_LIMITS"])
        return self
